﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Net;
using System.IO;

using System.Web; //Add reference to System.Web
using System.Runtime.Serialization.Json; //Add reference to System.Runtime.Serialization

namespace RedCritterConnecter.Samples
{
    //Create a class to contain the response
    public class AwardCertResponse
    {
        public String ErrorMessage { get; set; }
        public Boolean Submitted { get; set; }
        public String RequestID { get; set; }
        public String APIVersion { get; set; }
        public String ReturnParam1 { get; set; }
        public String ReturnParam2 { get; set; }
        public String ReturnParam3 { get; set; }
        private Int32 ErrorCode { get; set; }
    }

    public class AwardCert
    {

        public delegate void OnAwardCertResponse(AwardCertResponse AwardCertResponse);
        public event OnAwardCertResponse onAwardCertResponse;

        public delegate void OnAwardCertResponseError(AwardCertResponse AwardCertResponse);
        public event OnAwardCertResponseError onAwardCertResponseError;

        const string CONST_ConnecterBaseURL = "https://www.redcritterconnecter.com/";
        
        public void Execute(String SecretKey, String ProfileEntity, String CertName, String DateIssued, String ExpirationDate)
        {
            try
            {
                //Create url encoded parameters in query string
                String queryString = "secretkey=" + System.Web.HttpUtility.UrlEncode(SecretKey) + "&Certname=" + System.Web.HttpUtility.UrlEncode(CertName) + "&profileentity=" + System.Web.HttpUtility.UrlEncode(ProfileEntity) + "&dateissued=" + System.Web.HttpUtility.UrlEncode(DateIssued) + "&expirationdate=" + System.Web.HttpUtility.UrlEncode(ExpirationDate);

                //Create a new instance of a WebClient
                WebClient wc = new System.Net.WebClient();

                //Prevent this request from caching in order to ensure that it is sent to server
                wc.CachePolicy = new System.Net.Cache.RequestCachePolicy(System.Net.Cache.RequestCacheLevel.NoCacheNoStore);

                //Attach an event handler to receive the response
                wc.DownloadStringCompleted += onAwardCertResponseReceived;

                //Make the call
                wc.DownloadStringAsync(new Uri(CONST_ConnecterBaseURL + "services/gamificationv1/AwardCert?" + queryString, UriKind.Absolute));
            }
            catch
            {
                //Something went wrong communicating with the server
                if (onAwardCertResponseError != null)
                {
                    onAwardCertResponseError(null);
                }
            }

        }

        private void onAwardCertResponseReceived(object sender, DownloadStringCompletedEventArgs e)
        {
            try
            {
                //Create a JSON serializer
                System.Runtime.Serialization.Json.DataContractJsonSerializer s = new DataContractJsonSerializer(typeof(AwardCertResponse));

                //Copy the string into a memory stream
                MemoryStream ms = new MemoryStream(Encoding.UTF8.GetBytes(e.Result));

                //Read the stream into an object matching the response's type
                AwardCertResponse AwardCertResponse = (AwardCertResponse)s.ReadObject(ms);

                //Work with the populated response object
                if (AwardCertResponse.Submitted == true)
                {
                    //Request was accepted, raise the success event
                    onAwardCertResponse(AwardCertResponse);
                }
                else
                {
                    //Something went wrong
                    if (onAwardCertResponseError != null)
                    {
                        //Request failed, raise the error event
                        onAwardCertResponseError(AwardCertResponse);
                    }
                }
            }
            catch
            {
                //Something went wrong
                onAwardCertResponseError(null);
            }
        }
    }
}
